@@ -1,18 +1,29 @@
 from __future__ import annotations
 
-from typing import Any, Dict, List, Optional, Sequence, Type, Union
-
 import copy
 import glob
 import importlib
 import inspect
 import logging
 import os
-import pkg_resources
+from pathlib import Path
 import sys
 import traceback
 import typing
-from typing import TYPE_CHECKING
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Sequence,
+    Type,
+    Union,
+)
+
+import pkg_resources
+
 from policyengine_core import commons, periods, variables
 from policyengine_core.data_structures.reference import Reference
 from policyengine_core.entities import Entity
@@ -34,8 +45,7 @@ from policyengine_core.parameters.operations.uprate_parameters import (
     uprate_parameters,
 )
 from policyengine_core.periods import Instant, Period
-from policyengine_core.populations import Population, GroupPopulation
-from policyengine_core.tools.test_runner import run_tests
+from policyengine_core.populations import GroupPopulation, Population
 from policyengine_core.variables import Variable
 
 log = logging.getLogger(__name__)
@@ -62,6 +72,7 @@ class TaxBenefitSystem:
     baseline: "TaxBenefitSystem" = None  # Baseline tax-benefit system. Used only by reforms. Note: Reforms can be chained.
     cache_blacklist = None
     decomposition_file_path = None
+    variable_module_metadata: dict = None
 
     # The following properties should be specified by country packages.
 
@@ -71,6 +82,10 @@ class TaxBenefitSystem:
     """Directory containing the Python files defining the variables of the tax and benefit system."""
     parameters_dir: str = None
     """Directory containing the YAML parameter tree."""
+    auto_carry_over_input_variables: bool = False
+    """Whether to automatically carry over input variables when calculating a variable for a period different from the period of the input variables."""
+    basic_inputs: List[str] = None
+    """Short list of basic inputs to get medium accuracy."""
 
     def __init__(self, entities: Sequence[Entity] = None) -> None:
         if entities is None:
@@ -95,9 +110,12 @@ class TaxBenefitSystem:
         self.group_entities = [
             entity for entity in self.entities if not entity.is_person
         ]
+        self.group_entity_keys = [entity.key for entity in self.group_entities]
         for entity in self.entities:
             entity.set_tax_benefit_system(self)
 
+        self.variable_module_metadata = {}
+
         if self.variables_dir is not None:
             self.add_variables_from_directory(self.variables_dir)
 
@@ -106,6 +124,7 @@ class TaxBenefitSystem:
             self.parameters = homogenize_parameter_structures(
                 self.parameters, self.variables
             )
+            self.parameters = propagate_parameter_metadata(self.parameters)
             self.parameters = interpolate_parameters(self.parameters)
             self.parameters = uprate_parameters(self.parameters)
             self.parameters = propagate_parameter_metadata(self.parameters)
@@ -141,9 +160,7 @@ class TaxBenefitSystem:
         baseline_variable = self.get_variable(name)
         if baseline_variable and not update:
             raise VariableNameConflictError(
-                'Variable "{}" is already defined. Use `update_variable` to replace it.'.format(
-                    name
-                )
+                f"You've already defined {name} in {baseline_variable.module_name}. You tried to define it again in {variable_class.module_name}."
             )
 
         variable = variable_class(baseline_variable=baseline_variable)
@@ -199,6 +216,18 @@ class TaxBenefitSystem:
         try:
             file_name = os.path.splitext(os.path.basename(file_path))[0]
 
+            path = Path(file_path)
+
+            # Get the relative location, e.g. policyengine_uk/variables/gov/child_benefit.py -> gov.child_benefit
+            try:
+                relative_file_path = (
+                    str(path.relative_to(self.variables_dir))
+                    .replace("/", ".")
+                    .replace(".py", "")
+                )
+            except:
+                relative_file_path = ""
+
             #  As Python remembers loaded modules by name, in order to prevent collisions, we need to make sure that:
             #  - Files with the same name, but located in different directories, have a different module names. Hence the file path hash in the module name.
             #  - The same file, loaded by different tax and benefit systems, has distinct module names. Hence the `id(self)` in the module name.
@@ -221,9 +250,19 @@ class TaxBenefitSystem:
                 raise
             potential_variables = [
                 getattr(module, item)
-                for item in dir(module)
+                for item in module.__dict__
                 if not item.startswith("__")
             ]
+
+            metadata = {}
+            metadata["label"] = module.__dict__.get(
+                "label", relative_file_path
+            )
+            metadata["description"] = module.__dict__.get("description", None)
+            metadata["index"] = module.__dict__.get("index", 0)
+            self.variable_module_metadata[relative_file_path] = metadata
+
+            i = 0
             for pot_variable in potential_variables:
                 # We only want to get the module classes defined in this module (not imported)
                 if (
@@ -231,6 +270,9 @@ class TaxBenefitSystem:
                     and issubclass(pot_variable, Variable)
                     and pot_variable.__module__ == module_name
                 ):
+                    pot_variable.module_name = relative_file_path
+                    pot_variable.index_in_module = i
+                    i += 1
                     self.add_variable(pot_variable)
         except Exception:
             log.error(
@@ -240,11 +282,53 @@ class TaxBenefitSystem:
             )
             raise
 
+    def add_variable_metadata_from_folder(self, file_path: str) -> None:
+        """
+        Adds metadata from a given README.md file to the tax and benefit system.
+        """
+        try:
+            path = Path(file_path)
+
+            # Get the relative location, e.g. policyengine_uk/variables/gov/child_benefit.py -> gov.child_benefit
+            try:
+                relative_file_path = str(
+                    path.parent.relative_to(self.variables_dir)
+                ).replace("/", ".")
+            except:
+                relative_file_path = ""
+
+            metadata = {}
+
+            with open(file_path, "r") as f:
+                # Get the header as the label (making sure to remove the leading hash), and the rest as the description
+                lines = f.readlines()
+                metadata["label"] = lines[0].replace("# ", "").strip()
+                metadata["description"] = "".join(lines[1:]).strip()
+
+            self.variable_module_metadata[relative_file_path] = metadata
+        except Exception:
+            log.error(
+                'Unable to load OpenFisca metadata from file "{}"'.format(
+                    file_path
+                )
+            )
+            raise
+
     def add_variables_from_directory(self, directory: str) -> None:
         """
         Recursively explores a directory, and adds all OpenFisca variables found there to the tax and benefit system.
         """
         py_files = glob.glob(os.path.join(directory, "*.py"))
+        # Try to get the __init__.py file. The __init__.py file may contain metadata about the directory.
+        init_module = os.path.join(directory, "__init__.py")
+        if init_module in py_files:
+            # Extract the label and description from the __init__.py file
+            py_files.remove(init_module)
+            self.add_variable_metadata_from_folder(init_module)
+        if "README.md" in os.listdir(directory):
+            self.add_variable_metadata_from_folder(
+                os.path.join(directory, "README.md")
+            )
         for py_file in py_files:
             self.add_variables_from_file(py_file)
         subdirectories = glob.glob(os.path.join(directory, "*/"))
@@ -521,12 +605,16 @@ class TaxBenefitSystem:
                 "variables",
             ):
                 new_dict[key] = value
-        for entity in new_dict["entities"]:
-            entity.set_tax_benefit_system(new)
 
         new_dict["parameters"] = self.parameters.clone()
         new_dict["_parameters_at_instant_cache"] = {}
-        new_dict["variables"] = self.variables.copy()
+        new_dict["variables"] = {
+            variable_name: variable.clone()
+            for variable_name, variable in self.variables.items()
+        }
+
+        for entity in new_dict["entities"]:
+            entity.set_tax_benefit_system(new)
         return new
 
     def entities_plural(self) -> dict:
@@ -536,4 +624,29 @@ class TaxBenefitSystem:
         return {entity.key: entity for entity in self.entities}
 
     def test(self, paths: str, verbose: bool = False) -> None:
+        from policyengine_core.tools.test_runner import (
+            run_tests,
+        )  # Import here to avoid circular dependency.
+
         run_tests(self, paths, options=dict(verbose=verbose))
+
+    def modify_parameters(
+        self, modifier_function: Callable[[ParameterNode], ParameterNode]
+    ) -> None:
+        """Make modifications on the parameters of the legislation.
+
+        Call this function in `apply()` if the reform asks for legislation parameter modifications.
+
+        Args:
+            modifier_function: A function that takes a :obj:`.ParameterNode` and should return an object of the same type.
+        """
+        reform_parameters = modifier_function(self.parameters)
+        if not isinstance(reform_parameters, ParameterNode):
+            return ValueError(
+                "modifier_function {} in module {} must return a ParameterNode".format(
+                    modifier_function.__name__,
+                    modifier_function.__module__,
+                )
+            )
+        self.parameters = reform_parameters
+        self._parameters_at_instant_cache = {}
